/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2013-2014 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of the GNU General
 * Public License Version 2 only ("GPL"). You may not use this file except
 * in compliance with the License.  You can obtain a copy of the License at
 * https://avatar.java.net/license.html or legal/LICENSE.txt.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 */

/*****
 * @module org/glassfish/avatar-model
 */

'use strict';

// -- Model --

/**
 * <p>A Model is created by calling {@link module:org/glassfish/avatar-model.newModel}. The name
 * <em>Model</em> in this case refers to the name given by the application to the value returned
 * by this method. For example:
 * <pre>
 *     var Product = avatar.newModel('pair', {
 *      "x": "integer",
 *      "y": "integer
 *     });
 * </pre>
 * where <em>Model</em> is <tt>Product</tt>.
 * The model returned is also a constructor function, so it can be used to create instances
 * using <tt>new</tt>. However, in most cases, this is not really necessary since its
 * methods are all static.
 *
 * @param {object} obj Optional object literal used to initialize a model instance.
 * @interface Model
 * @class Model
 */
function Model(obj) {}

/**
 * Create a model instance and insert it into a store (if bound).
 *
 * @see {@link bind}
 * @memberOf Model
 * @param {object} obj Object used to initialize model instance.
 * @param {callback} cb Callback function to pass the model instance or error.
 * @returns {Promise} A promise for a model instance if no callback is specified.
 */
Model.create = function(obj, cb) {};

/**
 * Get a model instance.
 *
 * @memberOf Model
 * @param {...object} id One or more arguments that comprise the primary for the instance.
 * @param {callback} cb Callback function to pass the model instance or error.
 * @returns {Promise} A promise for a model instance (or null) if no callback is specified.
 */
Model.get = function(id, cb) {};

/**
 * Update a model model instance.
 *
 * @memberOf Model
 * @param {object} obj Object used to update model instance.
 * @param {callback} cb Callback function to pass the updated model instance or error.
 * @returns {Promise} A promise for an updated model instance if no callback is specified.
 */
Model.put = function(obj, cb) {};

/**
 * Delete a model instance using its primary key.
 *
 * @memberOf Model
 * @param {...object} id One or more arguments that comprise the primary for the instance.
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
Model.delete = function(id, cb) {};

/**
 * Get all instances of a model.
 *
 * @memberOf Model
 * @param {callback} cb Callback function to pass an array of model instances or error.
 * @returns {Promise} A promise for an array of model instances if no callback is specified.
 */
Model.getAll = function(cb) {};

/**
 * Delete all instances of a model.
 *
 * @memberOf Model
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
Model.deleteAll = function() {};

/**
 * Query instances of a model using a query language which depends on the store. JPA-based
 * stores will accept the JPQL languages; some stores may not support querying at all.
 *
 * @memberOf Model
 * @param {callback} cb Callback function to pass an array of model instances or error.
 * @returns {Promise} A promise for an array of model instances if no callback is specified.
 */
Model.query = function() {};

/**
 * Establishes a 1:N or parent-child relationship between this and another model.
 * A 1:N relationship is bi-directional, so defining a 1:N relationship between model
 * <tt>X</tt> and <tt>Y</tt>, automatically creates the inverse N:1 relationship
 * between model <tt>Y</tt> and </tt>X</tt>.
 *
 * @memberOf Model
 * @param {Model} model Reference to other model.
 * @param {object} properties Object literal that defines relationship.
 * @returns {undefined}
 * @example
 *  Family.hasMany(Product, {
 *      as: 'products',
 *      foreign: 'family'
 *  });
 */
Model.hasMany = function(model, properties) {};

/**
 * Establishes a 1:1 relationship between this and another model. A 1:1 relationship
 * is bi-directional, so defining a 1:1 relationship between model <tt>X</tt> and
 * <tt>Y</tt>, automatically creates the inverse 1:1 relationship between model
 * <tt>Y</tt> and </tt>X</tt>.
 * 
 * @memberOf Model
 * @param {Model} model Reference to other model.
 * @param {object} properties Object literal that defines relationship.
 * @returns {undefined}
 * @example
 *  Product.hasOne(Master, {
 *      as: 'master',
 *      foreign: 'product'
 *  });
 */
Model.hasOne = function(model, properties) {};

/**
 * Determines if a model is bound to a store.
 *
 * @memberOf Model
 * @returns {boolean} Outcome of test.
 */
Model.isBound = function() {};

/**
 * Determines if a model was defined using a schema.
 *
 * @memberOf Model
 * @returns {boolean} Outcome of test.
 */
Model.hasSchema = function() {} ;

/**
 * Binds this model to a store.
 * 
 * @memberOf Model
 * @returns {undefined}
 */
Model.bind = function() {} ;

/**
 * <p>The property name <em>as</em> in this case refers to the name given by the application
 * when defining a model relationship. For example:
 * <pre>
 *  Family.hasMany(Product, {
 *      as: 'products',
 *      foreign: 'family'
 *  });
 * </pre>
 * where <em>as</em> is <tt>products</tt> in this case.
 *
 * <p>The model referred to by this property is a <em>child</em> model, in a 1:N relationship,
 * a <em>parent</em> in the inverse N:1 relationship or a <em>related</em> in a 1:1 
 * relationship. These correspond to the types <tt>ChildModel</tt>, <tt>ParentModel</tt>
 * and <tt>RelatedModel</tt>, respectively. For example, given the relationship defined
 * above, the following snipped obtains all the products of a given family:
 * <pre>
 *  f.products.getAll().then(
 *      function(ps) { 
 *          ...
 *      });
 * </pre>
 *
 * @see {@link hasMany}
 * @see {@link hasOne}
 * @memberOf Model
 * @type {ChildModel|ParentModel|RelatedModel}
 */
Model.as = {};

// -- ParentModel --

/**
 * A ParentModel represents a model in the <em>left-hand-side</em> of a 1:N relationship.
 * A parent model owns its children, and can create, update or delete any of them.
 * In the following code:
 * <pre>
 *  Family.hasMany(Product, {
 *      as: 'products',
 *      foreign: 'family'
 *  });
 * </pre>
 * <tt>Family</tt> is a parent model and <tt>Product</tt> a child model.
 * 
 * @param {object} obj Optional object used to initialize the model instance.
 * @interface ParentModel
 * @class ParentModel
 * @see {@link ChildModel}
 */
function ParentModel(obj) {}

/**
 * Create a child model instance and insert it into a store (if bound).
 *
 * @see {@link bind}
 * @memberOf ParentModel
 * @param {object} obj Object used to initialize child model instance.
 * @param {callback} cb Callback function to pass the child model instance or error.
 * @returns {Promise} A promise for a child model instance if no callback is specified.
 */
ParentModel.create = function(obj, cb) {};

/**
 * Get a child model instance.
 *
 * @memberOf ParentModel
 * @param {...object} id One or more arguments that comprise the primary for the instance.
 * @param {callback} cb Callback function to pass the model instance or error.
 * @returns {Promise} A promise for a model instance (or null) if no callback is specified.
 * @see {@link Model.get} To retrieve model instances that are not related to this model.
 */
ParentModel.get = function(id, cb) {};

/**
 * Update a child model  instance.
 *
 * @memberOf ParentModel
 * @param {object} obj Object used to update child model instance.
 * @param {callback} cb Callback function to pass the updated child model instance or error.
 * @returns {Promise} A promise for an updated child model instance if no callback is specified.
 */
ParentModel.put = function(obj, cb) {};

/**
 * Delete a child model instance using its primary key.
 *
 * @memberOf ParentModel
 * @param {...object} id One or more arguments that comprise the primary for the instance.
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
ParentModel.delete = function(id, cb) {};

/**
 * Get all child instances of a model.
 *
 * @memberOf ParentModel
 * @param {callback} cb Callback function to pass an array of model instances or error.
 * @returns {Promise} A promise for an array of model instances if no callback is specified.
 */
ParentModel.getAll = function(cb) {};

/**
 * Delete all child instances of a model.
 *
 * @memberOf ParentModel
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
ParentModel.deleteAll = function(cb) {};

// -- ChildModel --

/**
 * A ChildModel represents a model in the <em>right-hand-side</em> of a 1:N relationship. 
 * A child model holds a reference to its parent model. In the following code:
 * <pre>
 *  Family.hasMany(Product, {
 *      as: 'products',
 *      foreign: 'family'
 *  });
 * </pre>
 * <tt>Family</tt> is a parent model and <tt>Product</tt> a child model.
 *
 * @param {object} obj Optional object used to initialize the model instance.
 * @interface ChildModel
 * @class ChildModel
 * @see {@link ParentModel}
 */
function ChildModel(obj) {}

/**
 * Get the parent model of a child model.
 *
 * @memberOf ChildModel
 * @param {callback} cb Callback function to pass the parent model instance or error.
 * @returns {Promise} A promise for a parent model instance if no callback is specified.
 */
ChildModel.get = function(cb) {};

/**
 * Update the parent model of a child model.
 *
 * @memberOf ChildModel
 * @param {object} obj Object used to update parent model instance.
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
ChildModel.put = function(obj, cb) {};

// -- RelatedModel --

/**
 * A RelatedModel represents a model in a 1:1 relationship. A related model holds a reference
 * to another model. In the following code:
 * <pre>
 *  Product.hasOne(Master, {
 *      as: 'master',
 *      foreign: 'product'
 *  });
 * </pre>
 * models <tt>Product</tt> and <tt>Master</tt> are related models.
 *
 * @param {object} obj Optional object used to initialize the model instance.
 * @interface RelatedModel
 * @class RelatedModel
 */
function RelatedModel(obj) {}

/**
 * Get the related model of this model.
 *
 * @memberOf RelatedModel
 * @param {callback} cb Callback function to pass the related model (or null) instance or error.
 * @returns {Promise} A promise for a related model instance (or null) if no callback is specified.
 */
RelatedModel.get = function(cb) {};

/**
 * Update the related model of this model.
 *
 * @memberOf RelatedModel
 * @param {object} model Model instance to refer. May be <tt>null</tt> to break a relationship.
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
RelatedModel.put = function(model, cb) {};

/**
 * Create a related model instance and refer to it.
 *
 * @see {@link bind}
 * @memberOf RelatedModel
 * @param {object} obj Object used to initialize related model instance.
 * @param {callback} cb Callback function to pass the related model instance or error.
 * @returns {Promise} A promise for a related model instance if no callback is specified.
 */
RelatedModel.create = function(obj, cb) {};

// -- Store --

/**
 * <p>A Store is created by calling {@link module:org/glassfish/avatar-model.newStore}. This
 * symbol is here exclusively to document objects returned by that method.
 *
 * @interface Store
 * @class Store
 */
function Store() {};

/**
 * Connect to the store.
 *
 * @memberOf Store
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
Store.connect = function(cb) {};

/**
 * Disconnect from the store.
 *
 * @memberOf Store
 * @param {callback} cb Callback function to pass undefined to signal completion or an error.
 * @returns {Promise} A promise for undefined to signal completion if no callback is specified.
 */
Store.disconnect = function(cb) {};

/**
 * Start a transaction with one or more tasks are executed in sequence. A task is a function
 * that returns a promise which is fulfilled when the task completes or rejected if the task
 * fails.
 *
 * @memberOf Store
 * @param {...function} task One or more tasks to execute in the transaction.
 * @returns {Promise} A promise for a {Transaction} object.
 */
Store.transactionSeq = function(task) {};

/**
 * Start a transaction with one or more tasks are executed in parallel. A task is a function
 * that returns a promise which is fulfilled when the task completes or rejected if the task
 * fails.
 *
 * @memberOf Store
 * @param {...function} task One or more tasks to execute in the transaction.
 * @returns {Promise} A promise for a {Transaction} object.
 */
Store.transactionPar = function(task) {};

/**
 * Start a transaction whose task is defined by a single function. A task is a function
 * that returns a promise which is fulfilled when the task completes or rejected if the task
 * fails.
 *
 * @memberOf Store
 * @param {function} task One or more tasks to execute in the transaction.
 * @returns {Promise} A promise for a {Transaction} object.
 */
Store.transaction = function(task) {};

/**
 * Bind one ore more models to this store.
 *
 * @memberOf Store
 * @param {...Model} model One or more models to bind to this store.
 * @returns {undefined}
 */
Store.bind = function(model) {};

/**
 * Get a reference to the current transaction object.
 *
 * @memberOf Store
 * @returns {Transaction} Current transaction object or <tt>null</tt>.
 */
Store.getTx = function() {};

// -- Transaction --

/**
 * <p>A Transaction is only created via a {@link Store}. This symbol is here exclusively to document
 * objects returned by methods in that class.
 *
 * @interface Transaction
 * @class Transaction
 */
function Transaction() {};

/**
 * Register a callback for the transaction committed event.
 *
 * @memberOf Transaction
 * @param {callback} cb Function called after a transaction is committed.
 * @returns {undefined}
 */
Transaction.onCommitted = function(cb) {};

/**
 * Register a callback for the transaction rolled back event.
 *
 * @memberOf Transaction
 * @param {callback} cb Function called after a transaction is rolled back.
 * @returns {undefined}
 */
Transaction.onRolledback = function(cb) {};

/**
 * Register a callback for the transaction committing event.
 *
 * @memberOf Transaction
 * @param {callback} cb Function called right before a transaction is committed.
 * @returns {undefined}
 */
Transaction.onCommitting = function(cb) {};

/**
 * Register a callback for the transaction rolling back event.
 *
 * @memberOf Transaction
 * @param {callback} cb Function called right before a transaction is rolled back.
 * @returns {undefined}
 */
Transaction.onRollingback = function(cb) {};
